@@ -2,6 +2,7 @@ from ._linalg import complete_basis, direct_sum, orthonormalize
 from ._normalize_activation import moment, normalize2mom
 from ._soft_unit_step import soft_unit_step
 from ._soft_one_hot_linspace import soft_one_hot_linspace
+from ._reduce import germinate_formulas, reduce_permutation
 
 
 __all__ = [
@@ -12,4 +13,6 @@ __all__ = [
     "normalize2mom",
     "soft_unit_step",
     "soft_one_hot_linspace",
+    "germinate_formulas",
+    "reduce_permutation"
 ]
