@@ -13,7 +13,7 @@ class Activation(torch.nn.Module):
 
     Parameters
     ----------
-    irreps_in : `Irreps`
+    irreps_in : `e3nn.o3.Irreps`
         representation of the input
 
     acts : list of function or None
