@@ -10,9 +10,9 @@ class Identity(torch.nn.Module):
 
     Parameters
     ----------
-    irreps_in : `Irreps`
+    irreps_in : `e3nn.o3.Irreps`
 
-    irreps_out : `Irreps`
+    irreps_out : `e3nn.o3.Irreps`
     """
     def __init__(self, irreps_in, irreps_out):
         super().__init__()
@@ -31,7 +31,9 @@ class Identity(torch.nn.Module):
     def __repr__(self):
         return f"{self.__class__.__name__}({self.irreps_in} -> {self.irreps_out})"
 
-    def forward(self, features):
+    def forward(
+            # pylint: disable=no-self-use
+            self, features):
         """evaluate
         """
         return features
