@@ -17,10 +17,11 @@ class SphericalHarmonicsAlphaBeta(torch.nn.Module):
 
     Parameters are identical to :meth:`e3nn.o3.spherical_harmonics_alpha_beta`.
     """
-    _ls_list: List[Tuple[int, int]]
+    normalization: str
+    _ls_list: List[int]
     _lmax: int
 
-    def __init__(self, l):
+    def __init__(self, l, normalization='integral'):
         super().__init__()
 
         if isinstance(l, o3.Irreps):
@@ -30,19 +31,29 @@ class SphericalHarmonicsAlphaBeta(torch.nn.Module):
         else:
             ls = list(l)
 
-        self._ls_list = [(1, l) for l in ls]
+        self._ls_list = ls
         self._lmax = max(ls)
         self.legendre = Legendre(ls)
+        self.normalization = normalization
 
     def forward(self, alpha: torch.Tensor, beta: torch.Tensor) -> torch.Tensor:
         y, z = beta.cos(), beta.sin()
         sha = spherical_harmonics_alpha(self._lmax, alpha.flatten())  # [z, m]
         shy = self.legendre(y.flatten(), z.flatten())  # [z, l * m]
-        out = _mul_m_lm(self._ls_list, sha, shy)
+        out = _mul_m_lm([(1, l) for l in self._ls_list], sha, shy)
+
+        if self.normalization == 'norm':
+            out.div_(torch.cat([
+                (math.sqrt(2 * l + 1) / math.sqrt(4 * math.pi)) * torch.ones(2 * l + 1, dtype=out.dtype, device=out.device)
+                for l in self._ls_list
+            ]))
+        elif self.normalization == 'component':
+            out.mul_(math.sqrt(4 * math.pi))
+
         return out.reshape(alpha.shape + (shy.shape[1],))
 
 
-def spherical_harmonics_alpha_beta(l, alpha, beta):
+def spherical_harmonics_alpha_beta(l, alpha, beta, *, normalization='integral'):
     r"""Spherical harmonics of :math:`\vec r = R_y(\alpha) R_x(\beta) e_y`
 
     .. math:: Y^l(\alpha, \beta) = S^l(\alpha) P^l(\cos(\beta))
@@ -66,7 +77,7 @@ def spherical_harmonics_alpha_beta(l, alpha, beta):
     `torch.Tensor`
         a tensor of shape ``(..., 2l+1)``
     """
-    sh = SphericalHarmonicsAlphaBeta(l)
+    sh = SphericalHarmonicsAlphaBeta(l, normalization=normalization)
     return sh(alpha, beta)
 
 
@@ -105,6 +116,8 @@ def spherical_harmonics_alpha(l: int, alpha: torch.Tensor) -> torch.Tensor:
 
 @compile_mode('script')
 class Legendre(fx.GraphModule):
+    # pylint: disable=abstract-method
+
     def __init__(self, ls):
         super().__init__(self, fx.Graph())
 
