@@ -2,7 +2,7 @@ import collections
 import torch
 from torch import fx
 from e3nn import o3
-from e3nn.math import group
+from e3nn.math import germinate_formulas, reduce_permutation
 from e3nn.util import explicit_default_types
 from e3nn.util.jit import compile_mode
 
@@ -32,7 +32,7 @@ def _wigner_nj(*irrepss, normalization='component', filter_ir_mid=None, dtype=No
 
     *irrepss_left, irreps_right = irrepss
     ret = []
-    for ir_left, path_left, C_left in _wigner_nj(*irrepss_left, dtype=dtype, device=device):
+    for ir_left, path_left, C_left in _wigner_nj(*irrepss_left, normalization=normalization, filter_ir_mid=filter_ir_mid, dtype=dtype, device=device):
         i = 0
         for mul, ir in irreps_right:
             for ir_out in ir_left * ir:
@@ -86,22 +86,22 @@ class ReducedTensorProducts(fx.GraphModule):
         For instance ``ij=ji`` means that the tensor has to indices and if they are exchanged, its value is the same.
         ``ij=-ji`` means that the tensor change its sign if the two indices are exchanged.
 
-    filter_ir_out : list of `Irrep`, optional
+    filter_ir_out : list of `e3nn.o3.Irrep`, optional
         Optional, list of allowed irrep in the output
 
-    filter_ir_mid : list of `Irrep`, optional
+    filter_ir_mid : list of `e3nn.o3.Irrep`, optional
         Optional, list of allowed irrep in the intermediary operations
 
-    **kwargs : dict of `Irreps`
+    **kwargs : dict of `e3nn.o3.Irreps`
         each letter present in the formula has to be present in the ``irreps`` dictionary, unless it can be inferred by the formula.
         For instance if the formula is ``ij=ji`` you can provide the representation of ``i`` only: ``ReducedTensorProducts('ij=ji', i='1o')``.
 
     Attributes
     ----------
-    irreps_in : list of `Irreps`
+    irreps_in : list of `e3nn.o3.Irreps`
         input representations
 
-    irreps_out : `Irreps`
+    irreps_out : `e3nn.o3.Irreps`
         output representation
 
     change_of_basis : `torch.Tensor`
@@ -125,13 +125,15 @@ class ReducedTensorProducts(fx.GraphModule):
     >>> b = tp(x, y)
     >>> assert torch.allclose(a, b, atol=1e-3, rtol=1e-3)
     """
+    # pylint: disable=abstract-method
+
     def __init__(self, formula, filter_ir_out=None, filter_ir_mid=None, eps=1e-9, **irreps):
         super().__init__(self, fx.Graph())
 
         if filter_ir_out is not None:
             filter_ir_out = [o3.Irrep(ir) for ir in filter_ir_out]
 
-        f0, formulas = group.germinate_formulas(formula)
+        f0, formulas = germinate_formulas(formula)
 
         irreps = {i: o3.Irreps(irs) for i, irs in irreps.items()}
 
@@ -157,7 +159,7 @@ class ReducedTensorProducts(fx.GraphModule):
             if i not in f0:
                 raise RuntimeError(f'index {i} has an irreps but does not appear in the fomula')
 
-        base_perm, _ = group.reduce_permutation(
+        base_perm, _ = reduce_permutation(
             f0,
             formulas,
             dtype=torch.float64,
@@ -194,7 +196,7 @@ class ReducedTensorProducts(fx.GraphModule):
                     torch.cat([RR, -RP], dim=1),
                     torch.cat([-RP.T, PP], dim=1)
                 ], dim=0)
-                eigenvalues, eigenvectors = torch.symeig(prob, eigenvectors=True)
+                eigenvalues, eigenvectors = torch.linalg.eigh(prob)
                 X = eigenvectors[:, eigenvalues < eps][:mul].T  # [solutions, multiplicity]
                 X = torch.linalg.qr(X, mode='r').R
                 for i, x in enumerate(X):
@@ -204,6 +206,10 @@ class ReducedTensorProducts(fx.GraphModule):
                         if x[j] > eps:
                             break
 
+                X[X.abs() < eps] = 0
+                X = sorted([[x.item() for x in line] for line in X])
+                X = torch.tensor(X, dtype=torch.float64)
+
                 Xs.append(X)
 
             for X in Xs:
