@@ -1,4 +1,4 @@
-from typing import Optional, List, Union
+from typing import Optional, List, Union, Iterator
 
 import torch
 import torch.fx
@@ -19,13 +19,13 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
 
     Parameters
     ----------
-    irreps_in1 : `Irreps`
+    irreps_in1 : `e3nn.o3.Irreps`
         Irreps for the first input.
 
-    irreps_in2 : `Irreps`
+    irreps_in2 : `e3nn.o3.Irreps`
         Irreps for the second input.
 
-    irreps_out : `Irreps`
+    irreps_out : `e3nn.o3.Irreps`
         Irreps for the output.
 
     instructions : list of tuple
@@ -54,7 +54,7 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
             \| x \| = \| y \| = 1 \Longrightarrow \| x \otimes y \| = 1
 
     internal_weights : bool
-        whether the `TensorProduct` contains its learnable weights as a parameter
+        whether the `e3nn.o3.TensorProduct` contains its learnable weights as a parameter
 
     shared_weights : bool
         whether the learnable weights are shared among the input's extra dimensions
@@ -257,7 +257,7 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
         del opt_defaults
 
         # Generate the actual tensor product code
-        graph_out, graph_right, wigners = codegen_tensor_product(
+        graphmod_out, graphmod_right = codegen_tensor_product(
             self.irreps_in1,
             self.in1_var,
             self.irreps_in2,
@@ -271,12 +271,10 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
             self._optimize_einsums
         )
         self._codegen_register({
-            "_compiled_main_out": graph_out,
-            "_compiled_main_right": graph_right
+            "_compiled_main_out": graphmod_out,
+            "_compiled_main_right": graphmod_right
         })
 
-        self._wigners = wigners
-
         # === Determine weights ===
         self.weight_numel = sum(prod(ins.path_shape) for ins in self.instructions if ins.has_weight)
 
@@ -287,24 +285,6 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
             # For TorchScript, there always has to be some kind of defined .weight
             self.register_buffer('weight', torch.Tensor())
 
-        # w3j
-        wigner_mats = []
-        for l_1, l_2, l_out in wigners:
-            wig = o3.wigner_3j(l_1, l_2, l_out)
-
-            if normalization == 'component':
-                wig *= (2 * l_out + 1) ** 0.5
-            if normalization == 'norm':
-                wig *= (2 * l_1 + 1) ** 0.5 * (2 * l_2 + 1) ** 0.5
-
-            wigner_mats.append(wig)
-
-        if len(wigner_mats) > 0:
-            self.register_buffer('_wigner_buf', torch.cat([w.reshape(-1) for w in wigner_mats]))
-        else:
-            # We register an empty buffer so that call signatures don't have to change
-            self.register_buffer('_wigner_buf', torch.Tensor())
-
         if self.irreps_out.dim > 0:
             output_mask = torch.cat([
                 torch.ones(mul * ir.dim)
@@ -405,7 +385,7 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
 
         with torch.autograd.profiler.record_function(self._profiling_str):
             real_weight = self._get_weights(weight)
-            return self._compiled_main_right(y, real_weight, self._wigner_buf)
+            return self._compiled_main_right(y, real_weight)
 
     def forward(self, x, y, weight: Optional[torch.Tensor] = None):
         r"""Evaluate :math:`w x \otimes y`.
@@ -435,7 +415,7 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
 
         with torch.autograd.profiler.record_function(self._profiling_str):
             real_weight = self._get_weights(weight)
-            return self._compiled_main_out(x, y, real_weight, self._wigner_buf)
+            return self._compiled_main_out(x, y, real_weight)
 
     def weight_view_for_instruction(
         self,
@@ -497,7 +477,6 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
                     yield ins_i, ins, this_weight
                 else:
                     yield this_weight
-        return
 
     def visualize(
         self,
@@ -506,7 +485,7 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
         aspect_ratio=1,
         ax=None
     ):  # pragma: no cover
-        r"""Visualize the connectivity of this `TensorProduct`
+        r"""Visualize the connectivity of this `e3nn.o3.TensorProduct`
 
         Parameters
         ----------
@@ -537,7 +516,7 @@ class TensorProduct(CodeGenMixin, torch.nn.Module):
         import matplotlib
         import matplotlib.pyplot as plt
         from matplotlib.path import Path
-        import matplotlib.patches as patches
+        from matplotlib import patches
 
         if ax is None:
             ax = plt.gca()
@@ -706,23 +685,23 @@ class FullyConnectedTensorProduct(TensorProduct):
 
     Parameters
     ----------
-    irreps_in1 : `Irreps`
+    irreps_in1 : `e3nn.o3.Irreps`
         representation of the first input
 
-    irreps_in2 : `Irreps`
+    irreps_in2 : `e3nn.o3.Irreps`
         representation of the second input
 
-    irreps_out : `Irreps`
+    irreps_out : `e3nn.o3.Irreps`
         representation of the output
 
     normalization : {'component', 'norm'}
-        see `TensorProduct`
+        see `e3nn.o3.TensorProduct`
 
     internal_weights : bool
-        see `TensorProduct`
+        see `e3nn.o3.TensorProduct`
 
     shared_weights : bool
-        see `TensorProduct`
+        see `e3nn.o3.TensorProduct`
     """
     def __init__(
         self,
@@ -757,17 +736,17 @@ class ElementwiseTensorProduct(TensorProduct):
 
     Parameters
     ----------
-    irreps_in1 : `Irreps`
+    irreps_in1 : `e3nn.o3.Irreps`
         representation of the first input
 
-    irreps_in2 : `Irreps`
+    irreps_in2 : `e3nn.o3.Irreps`
         representation of the second input
 
-    filter_ir_out : iterator of `Irrep`, optional
-        filter to select only specific `Irrep` of the output
+    filter_ir_out : iterator of `e3nn.o3.Irrep`, optional
+        filter to select only specific `e3nn.o3.Irrep` of the output
 
     normalization : {'component', 'norm'}
-        see `TensorProduct`
+        see `e3nn.o3.TensorProduct`
 
     Examples
     --------
@@ -839,23 +818,23 @@ class FullTensorProduct(TensorProduct):
 
     Parameters
     ----------
-    irreps_in1 : `Irreps`
+    irreps_in1 : `e3nn.o3.Irreps`
         representation of the first input
 
-    irreps_in2 : `Irreps`
+    irreps_in2 : `e3nn.o3.Irreps`
         representation of the second input
 
-    filter_ir_out : iterator of `Irrep`, optional
-        filter to select only specific `Irrep` of the output
+    filter_ir_out : iterator of `e3nn.o3.Irrep`, optional
+        filter to select only specific `e3nn.o3.Irrep` of the output
 
     normalization : {'component', 'norm'}
-        see `TensorProduct`
+        see `e3nn.o3.TensorProduct`
     """
     def __init__(
         self,
-        irreps_in1,
-        irreps_in2,
-        filter_ir_out=None,
+        irreps_in1: o3.Irreps,
+        irreps_in2: o3.Irreps,
+        filter_ir_out: Iterator[o3.Irrep] = None,
         **kwargs
     ):
 
