@@ -1,18 +1,19 @@
-from typing import Iterable, Optional
-
 import random
 import math
-import itertools
-import warnings
-import logging
 import inspect
+import itertools
+import logging
+from typing import Iterable, Optional
+import warnings
 
+import numpy as np
 import torch
 
 from e3nn import o3
 from e3nn.util.jit import compile, get_tracing_inputs, get_compile_mode, _MAKE_TRACING_INPUTS
 from ._argtools import _get_args_in, _get_io_irreps, _transform, _rand_args
 
+# pylint: disable=unused-variable
 
 # Make a logger for reporting error statistics
 logger = logging.getLogger(__name__)
@@ -90,9 +91,9 @@ def random_irreps(
         len_max : int, optional
             The largest number of irreps to generate, defaults to 4.
         clean : bool, optional
-            If ``True``, only ``o3.Irreps`` objects will be returned. If ``False`` (the default), ``Irreps``-like objects like strings and lists of tuples can be returned.
+            If ``True``, only ``o3.Irreps`` objects will be returned. If ``False`` (the default), ``e3nn.o3.Irreps``-like objects like strings and lists of tuples can be returned.
         allow_empty : bool, optional
-            Whether to allow generating empty ``Irreps``.
+            Whether to allow generating empty ``e3nn.o3.Irreps``.
     Returns
     -------
         An irreps-like object if ``n == 1`` or a list of them if ``n > 1``
@@ -243,9 +244,9 @@ def equivariance_error(
         the function to test
     args_in : list
         the original inputs to pass to ``func``.
-    irreps_in : list of `Irreps` or `Irreps`
+    irreps_in : list of `e3nn.o3.Irreps` or `e3nn.o3.Irreps`
         the input irreps for each of the arguments in ``args_in``. If left as the default of ``None``, ``get_io_irreps`` will be used to try to infer them. If a sequence is provided, valid elements are also the string ``'cartesian'``, which denotes that the corresponding input should be dealt with as cartesian points in 3D, and ``None``, which indicates that the argument should not be transformed.
-    irreps_out : list of `Irreps` or `Irreps`
+    irreps_out : list of `e3nn.o3.Irreps` or `e3nn.o3.Irreps`
         the out irreps for each of the return values of ``func``. Accepts similar values to ``irreps_in``.
     ntrials : int
         run this many trials with random transforms
@@ -261,11 +262,11 @@ def equivariance_error(
     irreps_in, irreps_out = _get_io_irreps(func, irreps_in=irreps_in, irreps_out=irreps_out)
 
     if do_parity:
-        parity_ks = torch.Tensor([0, 1])
+        parity_ks = [0, 1]
     else:
-        parity_ks = torch.Tensor([0])
+        parity_ks = [0]
 
-    if ('cartesian_points' not in irreps_in):
+    if 'cartesian_points' not in irreps_in:
         # There's nothing to translate
         do_translation = False
     if do_translation:
@@ -296,12 +297,12 @@ def equivariance_error(
             x2 = func(*args_in)
 
             # Deal with output shapes
-            assert type(x1) == type(x2), f"Inconsistant return types {type(x1)} and {type(x2)}"
+            assert type(x1) == type(x2), f"Inconsistant return types {type(x1)} and {type(x2)}"  # pylint: disable=unidiomatic-typecheck
             if isinstance(x1, torch.Tensor):
                 # Make sequences
                 x1 = [x1]
                 x2 = [x2]
-            elif isinstance(x1, list) or isinstance(x1, tuple):
+            elif isinstance(x1, (list, tuple)):
                 # They're already tuples
                 x1 = list(x1)
                 x2 = list(x2)
@@ -504,3 +505,10 @@ def assert_normalized(
                 max_componentwise
             )
             assert max_componentwise <= atol, f"< x_i^2 > !~= {target:.6f} for output irrep #{i}, {irreps[i]}. Max componentwise error: {max_componentwise:.6f}"
+
+
+def set_random_seeds():
+    """Set the random seeds to try to get some reproducibility"""
+    torch.manual_seed(0)
+    random.seed(0)
+    np.random.seed(0)
